@@ -1,54 +1,32 @@
 # -*- coding: utf-8 -*-
 """
-Natsort can sort strings with numbers in a natural order.
-It provides the natsorted function to sort strings with
-arbitrary numbers.
+Along with ns_enum.py, this module contains all of the
+natsort public API.
 
-You can mix types with natsorted.  This can get around the new
-'unorderable types' issue with Python 3. Natsort will recursively
-descend into lists of lists so you can sort by the sublist contents.
-
-See the README or the natsort homepage for more details.
+The majority of the "work" is defined in utils.py.
 """
-from __future__ import (
-    print_function,
-    division,
-    unicode_literals,
-    absolute_import
-)
 
-# Std lib. imports.
-import re
-from operator import itemgetter
+import platform
 from functools import partial
-from warnings import warn
+from operator import itemgetter
 
-# Local imports.
-from natsort.ns_enum import ns
-from natsort.compat.py23 import u_format
-from natsort.utils import (
-    _natsort_key,
-    _args_to_enum,
-    _do_decoding,
-)
-
-# Make sure the doctest works for either python2 or python3
-__doc__ = u_format(__doc__)
+import natsort.compat.locale
+from natsort import utils
+from natsort.ns_enum import NS_DUMB, ns
 
 
-@u_format
 def decoder(encoding):
     """
     Return a function that can be used to decode bytes to unicode.
 
     Parameters
     ----------
-    encoding: str
+    encoding : str
         The codec to use for decoding. This must be a valid unicode codec.
 
     Returns
     -------
-    decode_function:
+    decode_function
         A function that takes a single argument and attempts to decode
         it using the supplied codec. Any `UnicodeErrors` are raised.
         If the argument was not of `bytes` type, it is simply returned
@@ -67,30 +45,28 @@ def decoder(encoding):
         True
         >>> f(12345) == 12345
         True
+        >>> # On Python 3, without decoder this would return [b'a10', b'a2']
         >>> natsorted([b'a10', b'a2'], key=decoder('utf8')) == [b'a2', b'a10']
         True
-        >>> # On Python 3, without decoder this would return [b'a10', b'a2']
+        >>> # On Python 3, without decoder this would raise a TypeError.
         >>> natsorted([b'a10', 'a2'], key=decoder('utf8')) == ['a2', b'a10']
         True
-        >>> # On Python 3, without decoder this would raise a TypeError.
 
     """
-    return partial(_do_decoding, encoding=encoding)
+    return partial(utils.do_decoding, encoding=encoding)
 
 
-@u_format
 def as_ascii(s):
     """
     Function to decode an input with the ASCII codec, or return as-is.
 
     Parameters
     ----------
-    s:
-        Any object.
+    s : object
 
     Returns
     -------
-    output:
+    output
         If the input was of type `bytes`, the return value is a `str` decoded
         with the ASCII codec. Otherwise, the return value is identically the
         input.
@@ -100,22 +76,20 @@ def as_ascii(s):
     decoder
 
     """
-    return _do_decoding(s, 'ascii')
+    return utils.do_decoding(s, "ascii")
 
 
-@u_format
 def as_utf8(s):
     """
     Function to decode an input with the UTF-8 codec, or return as-is.
 
     Parameters
     ----------
-    s:
-        Any object.
+    s : object
 
     Returns
     -------
-    output:
+    output
         If the input was of type `bytes`, the return value is a `str` decoded
         with the UTF-8 codec. Otherwise, the return value is identically the
         input.
@@ -125,28 +99,19 @@ def as_utf8(s):
     decoder
 
     """
-    return _do_decoding(s, 'utf-8')
+    return utils.do_decoding(s, "utf-8")
 
 
-def natsort_key(val, key=None, alg=0, **_kwargs):
-    """Undocumented, kept for backwards-compatibility."""
-    msg = "natsort_key is deprecated as of 3.4.0, please use natsort_keygen"
-    warn(msg, DeprecationWarning)
-    return _natsort_key(val, key, _args_to_enum(**_kwargs) | alg)
-
-
-@u_format
-def natsort_keygen(key=None, alg=0, **_kwargs):
-    """\
+def natsort_keygen(key=None, alg=ns.DEFAULT):
+    """
     Generate a key to sort strings and numbers naturally.
 
-    Generate a key to sort strings and numbers naturally,
-    not lexicographically. This key is designed for use as the
-    `key` argument to functions such as the `sorted` builtin.
+    This key is designed for use as the `key` argument to
+    functions such as the `sorted` builtin.
 
     The user may customize the generated function with the
     arguments to `natsort_keygen`, including an optional
-    `key` function which will be called before the `natsort_key`.
+    `key` function.
 
     Parameters
     ----------
@@ -163,45 +128,101 @@ def natsort_keygen(key=None, alg=0, **_kwargs):
     Returns
     -------
     out : function
-        A wrapped version of the `natsort_key` function that is
+        A function that parses input for natural sorting that is
         suitable for passing as the `key` argument to functions
         such as `sorted`.
 
     See Also
     --------
     natsorted
+    natsort_key
 
     Examples
     --------
     `natsort_keygen` is a convenient way to create a custom key
-    to sort lists in-place (for example). Calling with no objects
-    will return a plain `natsort_key` instance::
+    to sort lists in-place (for example).::
 
         >>> a = ['num5.10', 'num-3', 'num5.3', 'num2']
         >>> a.sort(key=natsort_keygen(alg=ns.REAL))
         >>> a
-        [{u}'num-3', {u}'num2', {u}'num5.10', {u}'num5.3']
+        ['num-3', 'num2', 'num5.10', 'num5.3']
 
     """
-    return partial(_natsort_key, key=key, alg=_args_to_enum(**_kwargs) | alg)
+    try:
+        ns.DEFAULT | alg
+    except TypeError:
+        msg = "natsort_keygen: 'alg' argument must be from the enum 'ns'"
+        raise ValueError(msg + ", got {}".format(str(alg)))
+
+    # Add the NS_DUMB option if the locale library is broken.
+    if alg & ns.LOCALEALPHA and natsort.compat.locale.dumb_sort():
+        alg |= NS_DUMB
+
+    # Set some variables that will be passed to the factory functions
+    if alg & ns.NUMAFTER:
+        if alg & ns.LOCALEALPHA:
+            sep = natsort.compat.locale.null_string_locale_max
+        else:
+            sep = natsort.compat.locale.null_string_max
+        pre_sep = natsort.compat.locale.null_string_max
+    else:
+        if alg & ns.LOCALEALPHA:
+            sep = natsort.compat.locale.null_string_locale
+        else:
+            sep = natsort.compat.locale.null_string
+        pre_sep = natsort.compat.locale.null_string
+    regex = utils.regex_chooser(alg)
+
+    # Create the functions that will be used to split strings.
+    input_transform = utils.input_string_transform_factory(alg)
+    component_transform = utils.string_component_transform_factory(alg)
+    final_transform = utils.final_data_transform_factory(alg, sep, pre_sep)
+
+    # Create the high-level parsing functions for strings, bytes, and numbers.
+    string_func = utils.parse_string_factory(
+        alg, sep, regex.split, input_transform, component_transform, final_transform
+    )
+    if alg & ns.PATH:
+        string_func = utils.parse_path_factory(string_func)
+    bytes_func = utils.parse_bytes_factory(alg)
+    num_func = utils.parse_number_or_none_factory(alg, sep, pre_sep)
+
+    # Return the natsort key with the parsing path pre-chosen.
+    return partial(
+        utils.natsort_key,
+        key=key,
+        string_func=string_func,
+        bytes_func=bytes_func,
+        num_func=num_func,
+    )
 
 
-@u_format
-def natsorted(seq, key=None, reverse=False, alg=0, **_kwargs):
-    """\
-    Sorts a sequence naturally.
+# Exposed for simplicity if one needs the default natsort key.
+natsort_key = natsort_keygen()
+natsort_key.__doc__ = """\
+natsort_key(val)
+The default natural sorting key.
 
-    Sorts a sequence naturally (alphabetically and numerically),
-    not lexicographically. Returns a new copy of the sorted
-    sequence as a list.
+This is the output of :func:`natsort_keygen` with default values.
+
+See Also
+--------
+natsort_keygen
+
+"""
+
+
+def natsorted(seq, key=None, reverse=False, alg=ns.DEFAULT):
+    """
+    Sorts an iterable naturally.
 
     Parameters
     ----------
     seq : iterable
-        The sequence to sort.
+        The input to sort.
 
     key : callable, optional
-        A key used to determine how to sort each element of the sequence.
+        A key used to determine how to sort each element of the iterable.
         It is **not** applied recursively.
         It should accept a single argument and return a single value.
 
@@ -217,7 +238,7 @@ def natsorted(seq, key=None, reverse=False, alg=0, **_kwargs):
     Returns
     -------
     out: list
-        The sorted sequence.
+        The sorted input.
 
     See Also
     --------
@@ -225,6 +246,7 @@ def natsorted(seq, key=None, reverse=False, alg=0, **_kwargs):
     realsorted : A wrapper for ``natsorted(seq, alg=ns.REAL)``.
     humansorted : A wrapper for ``natsorted(seq, alg=ns.LOCALE)``.
     index_natsorted : Returns the sorted indexes from `natsorted`.
+    os_sorted : Sort according to your operating system's rules.
 
     Examples
     --------
@@ -232,67 +254,23 @@ def natsorted(seq, key=None, reverse=False, alg=0, **_kwargs):
 
         >>> a = ['num3', 'num5', 'num2']
         >>> natsorted(a)
-        [{u}'num2', {u}'num3', {u}'num5']
+        ['num2', 'num3', 'num5']
 
     """
-    alg = _args_to_enum(**_kwargs) | alg
-    try:
-        return sorted(seq, reverse=reverse, key=natsort_keygen(key, alg=alg))
-    except TypeError as e:  # pragma: no cover
-        # In the event of an unresolved "unorderable types" error
-        # for string to number type comparisons (not str/bytes),
-        # attempt to sort again, being careful to prevent this error.
-        r = re.compile(r'(?:str|bytes)\(\) [<>] (?:str|bytes)\(\)')
-        if 'unorderable types' in str(e) and not r.search(str(e)):
-            return sorted(seq, reverse=reverse,
-                          key=natsort_keygen(key,
-                                             alg=alg | ns.TYPESAFE))
-        else:
-            # Re-raise if the problem was not "unorderable types"
-            raise
+    key = natsort_keygen(key, alg)
+    return sorted(seq, reverse=reverse, key=key)
 
 
-@u_format
-def versorted(seq, key=None, reverse=False, alg=0, **_kwargs):
-    """\
-    Identical to :func:`natsorted`.
-
-    This function exists for backwards compatibility with `natsort`
-    version < 4.0.0. Future development should use :func:`natsorted`.
-
-    Please see the :func:`natsorted` documentation for use.
-
-    See Also
-    --------
-    natsorted
-
+def humansorted(seq, key=None, reverse=False, alg=ns.DEFAULT):
     """
-    return natsorted(seq, key, reverse, alg, **_kwargs)
-
-
-@u_format
-def humansorted(seq, key=None, reverse=False, alg=0):
-    """\
     Convenience function to properly sort non-numeric characters.
 
-    Convenience function to properly sort non-numeric characters
-    in a locale-aware fashion (a.k.a "human sorting"). This is a
-    wrapper around ``natsorted(seq, alg=ns.LOCALE)``.
-
-    .. warning:: On BSD-based systems (like Mac OS X), the underlying
-                 C library that Python's locale module uses is broken.
-                 On these systems it is recommended that you install
-                 `PyICU <https://pypi.python.org/pypi/PyICU>`_
-                 if you wish to use ``humansorted``, especially if you need
-                 to handle non-ASCII characters.  If you are on
-                 one of systems and get unexpected results, please try
-                 using `PyICU <https://pypi.python.org/pypi/PyICU>`_
-                 before filing a bug report to `natsort`.
+    This is a wrapper around ``natsorted(seq, alg=ns.LOCALE)``.
 
     Parameters
     ----------
     seq : iterable
-        The sequence to sort.
+        The input to sort.
 
     key : callable, optional
         A key used to determine how to sort each element of the sequence.
@@ -311,7 +289,7 @@ def humansorted(seq, key=None, reverse=False, alg=0):
     Returns
     -------
     out : list
-        The sorted sequence.
+        The sorted input.
 
     See Also
     --------
@@ -319,23 +297,7 @@ def humansorted(seq, key=None, reverse=False, alg=0):
 
     Notes
     -----
-    You may find that if you do not explicitly set
-    the locale your results may not be as you expect, although
-    as of ``natsort`` version 4.0.0 the sorting algorithm has been
-    updated to account for a buggy ``locale`` installation.
-    In the below example 'en_US.UTF-8' is used, but you should use your
-    locale::
-
-        >>> import locale
-        >>> # The 'str' call is only to get around a bug on Python 2.x
-        >>> # where 'setlocale' does not expect unicode strings (ironic,
-        >>> # right?)
-        >>> locale.setlocale(locale.LC_ALL, str('en_US.UTF-8'))
-        'en_US.UTF-8'
-
-    It is preferred that you do this before importing `natsort`.
-    If you use `PyICU <https://pypi.python.org/pypi/PyICU>`_ (see warning
-    above) then you should not need to do explicitly set a locale.
+    Please read :ref:`locale_issues` before using `humansorted`.
 
     Examples
     --------
@@ -343,21 +305,19 @@ def humansorted(seq, key=None, reverse=False, alg=0):
 
         >>> a = ['Apple', 'Banana', 'apple', 'banana']
         >>> natsorted(a)
-        [{u}'Apple', {u}'Banana', {u}'apple', {u}'banana']
+        ['Apple', 'Banana', 'apple', 'banana']
         >>> humansorted(a)
-        [{u}'apple', {u}'Apple', {u}'banana', {u}'Banana']
+        ['apple', 'Apple', 'banana', 'Banana']
 
     """
     return natsorted(seq, key, reverse, alg | ns.LOCALE)
 
 
-@u_format
-def realsorted(seq, key=None, reverse=False, alg=0):
-    """\
+def realsorted(seq, key=None, reverse=False, alg=ns.DEFAULT):
+    """
     Convenience function to properly sort signed floats.
 
-    Convenience function to properly sort signed floats within
-    strings (i.e. "a-5.7"). This is a wrapper around
+    A signed float in a string could be "a-5.7". This is a wrapper around
     ``natsorted(seq, alg=ns.REAL)``.
 
     The behavior of :func:`realsorted` for `natsort` version >= 4.0.0
@@ -367,7 +327,7 @@ def realsorted(seq, key=None, reverse=False, alg=0):
     Parameters
     ----------
     seq : iterable
-        The sequence to sort.
+        The input to sort.
 
     key : callable, optional
         A key used to determine how to sort each element of the sequence.
@@ -386,7 +346,7 @@ def realsorted(seq, key=None, reverse=False, alg=0):
     Returns
     -------
     out : list
-        The sorted sequence.
+        The sorted input.
 
     See Also
     --------
@@ -398,28 +358,27 @@ def realsorted(seq, key=None, reverse=False, alg=0):
 
         >>> a = ['num5.10', 'num-3', 'num5.3', 'num2']
         >>> natsorted(a)
-        [{u}'num2', {u}'num5.3', {u}'num5.10', {u}'num-3']
+        ['num2', 'num5.3', 'num5.10', 'num-3']
         >>> realsorted(a)
-        [{u}'num-3', {u}'num2', {u}'num5.10', {u}'num5.3']
+        ['num-3', 'num2', 'num5.10', 'num5.3']
 
     """
     return natsorted(seq, key, reverse, alg | ns.REAL)
 
 
-@u_format
-def index_natsorted(seq, key=None, reverse=False, alg=0, **_kwargs):
-    """\
-    Return the list of the indexes used to sort the input sequence.
+def index_natsorted(seq, key=None, reverse=False, alg=ns.DEFAULT):
+    """
+    Determine the list of the indexes used to sort the input sequence.
 
     Sorts a sequence naturally, but returns a list of sorted the
-    indexes and not the sorted list. This list of indexes can be
-    used to sort multiple lists by the sorted order of the given
-    sequence.
+    indexes and not the sorted list itself. This list of indexes
+    can be used to sort multiple lists by the sorted order of the
+    given sequence.
 
     Parameters
     ----------
     seq : iterable
-        The sequence to sort.
+        The input to sort.
 
     key : callable, optional
         A key used to determine how to sort each element of the sequence.
@@ -438,7 +397,7 @@ def index_natsorted(seq, key=None, reverse=False, alg=0, **_kwargs):
     Returns
     -------
     out : tuple
-        The ordered indexes of the sequence.
+        The ordered indexes of the input.
 
     See Also
     --------
@@ -458,73 +417,32 @@ def index_natsorted(seq, key=None, reverse=False, alg=0, **_kwargs):
         [2, 0, 1]
         >>> # Sort both lists by the sort order of a
         >>> order_by_index(a, index)
-        [{u}'num2', {u}'num3', {u}'num5']
+        ['num2', 'num3', 'num5']
         >>> order_by_index(b, index)
-        [{u}'baz', {u}'foo', {u}'bar']
+        ['baz', 'foo', 'bar']
 
     """
-    alg = _args_to_enum(**_kwargs) | alg
     if key is None:
         newkey = itemgetter(1)
     else:
+
         def newkey(x):
             return key(itemgetter(1)(x))
+
     # Pair the index and sequence together, then sort by element
-    index_seq_pair = [[x, y] for x, y in enumerate(seq)]
-    try:
-        index_seq_pair.sort(reverse=reverse,
-                            key=natsort_keygen(newkey, alg=alg))
-    except TypeError as e:  # pragma: no cover
-        # In the event of an unresolved "unorderable types" error
-        # attempt to sort again, being careful to prevent this error.
-        if 'unorderable types' in str(e):
-            index_seq_pair.sort(reverse=reverse,
-                                key=natsort_keygen(newkey,
-                                                   alg=alg | ns.TYPESAFE))
-        else:
-            # Re-raise if the problem was not "unorderable types"
-            raise
+    index_seq_pair = [(x, y) for x, y in enumerate(seq)]
+    index_seq_pair.sort(reverse=reverse, key=natsort_keygen(newkey, alg))
     return [x for x, _ in index_seq_pair]
 
 
-@u_format
-def index_versorted(seq, key=None, reverse=False, alg=0, **_kwargs):
-    """\
-    Identical to :func:`index_natsorted`.
-
-    This function exists for backwards compatibility with
-    ``index_natsort`` version < 4.0.0. Future development should use
-    :func:`index_natsorted`.
-
-    Please see the :func:`index_natsorted` documentation for use.
-
-    See Also
-    --------
-    index_natsorted
-
+def index_humansorted(seq, key=None, reverse=False, alg=ns.DEFAULT):
     """
-    return index_natsorted(seq, key, reverse, alg, **_kwargs)
-
-
-@u_format
-def index_humansorted(seq, key=None, reverse=False, alg=0):
-    """\
-    Return the list of the indexes used to sort the input sequence
-    in a locale-aware manner.
-
-    Sorts a sequence in a locale-aware manner, but returns a list
-    of sorted the indexes and not the sorted list. This list of
-    indexes can be used to sort multiple lists by the sorted order
-    of the given sequence.
-
     This is a wrapper around ``index_natsorted(seq, alg=ns.LOCALE)``.
-    Please see the ``humansorted`` documentation for caveats of
-    using ``index_humansorted``.
 
     Parameters
     ----------
     seq: iterable
-        The sequence to sort.
+        The input to sort.
 
     key: callable, optional
         A key used to determine how to sort each element of the sequence.
@@ -543,7 +461,7 @@ def index_humansorted(seq, key=None, reverse=False, alg=0):
     Returns
     -------
     out : tuple
-        The ordered indexes of the sequence.
+        The ordered indexes of the input.
 
     See Also
     --------
@@ -552,23 +470,7 @@ def index_humansorted(seq, key=None, reverse=False, alg=0):
 
     Notes
     -----
-    You may find that if you do not explicitly set
-    the locale your results may not be as you expect, although
-    as of ``natsort`` version 4.0.0 the sorting algorithm has been
-    updated to account for a buggy ``locale`` installation.
-    In the below example 'en_US.UTF-8' is used, but you should use your
-    locale::
-
-        >>> import locale
-        >>> # The 'str' call is only to get around a bug on Python 2.x
-        >>> # where 'setlocale' does not expect unicode strings (ironic,
-        >>> # right?)
-        >>> locale.setlocale(locale.LC_ALL, str('en_US.UTF-8'))
-        'en_US.UTF-8'
-
-    It is preferred that you do this before importing `natsort`.
-    If you use `PyICU <https://pypi.python.org/pypi/PyICU>`_ (see warning
-    above) then you should not need to explicitly set a locale.
+    Please read :ref:`locale_issues` before using `humansorted`.
 
     Examples
     --------
@@ -582,27 +484,14 @@ def index_humansorted(seq, key=None, reverse=False, alg=0):
     return index_natsorted(seq, key, reverse, alg | ns.LOCALE)
 
 
-@u_format
-def index_realsorted(seq, key=None, reverse=False, alg=0):
-    """\
-    Return the list of the indexes used to sort the input sequence
-    in a locale-aware manner.
-
-    Sorts a sequence in a locale-aware manner, but returns a list
-    of sorted the indexes and not the sorted list. This list of
-    indexes can be used to sort multiple lists by the sorted order
-    of the given sequence.
-
+def index_realsorted(seq, key=None, reverse=False, alg=ns.DEFAULT):
+    """
     This is a wrapper around ``index_natsorted(seq, alg=ns.REAL)``.
 
-    The behavior of :func:`index_realsorted` in `natsort` version >= 4.0.0
-    was the default behavior of :func:`index_natsorted` for `natsort`
-    version < 4.0.0.
-
     Parameters
     ----------
     seq: iterable
-        The sequence to sort.
+        The input to sort.
 
     key: callable, optional
         A key used to determine how to sort each element of the sequence.
@@ -621,7 +510,7 @@ def index_realsorted(seq, key=None, reverse=False, alg=0):
     Returns
     -------
     out : tuple
-        The ordered indexes of the sequence.
+        The ordered indexes of the input.
 
     See Also
     --------
@@ -640,12 +529,12 @@ def index_realsorted(seq, key=None, reverse=False, alg=0):
     return index_natsorted(seq, key, reverse, alg | ns.REAL)
 
 
-@u_format
+# noinspection PyShadowingBuiltins,PyUnresolvedReferences
 def order_by_index(seq, index, iter=False):
-    """\
+    """
     Order a given sequence by an index sequence.
 
-    The output of `index_natsorted` and `index_versorted` is a
+    The output of `index_natsorted` is a
     sequence of integers (index) that correspond to how its input
     sequence **would** be sorted. The idea is that this index can
     be used to reorder multiple sequences by the sorted order of the
@@ -654,29 +543,28 @@ def order_by_index(seq, index, iter=False):
 
     Parameters
     ----------
-    seq : iterable
+    seq : sequence
         The sequence to order.
 
     index : iterable
-        The sequence that indicates how to order `seq`.
+        The iterable that indicates how to order `seq`.
         It should be the same length as `seq` and consist
         of integers only.
 
     iter : {{True, False}}, optional
         If `True`, the ordered sequence is returned as a
-        generator expression; otherwise it is returned as a
+        iterator; otherwise it is returned as a
         list. The default is `False`.
 
     Returns
     -------
-    out : {{list, generator}}
-        The sequence ordered by `index`, as a `list` or as a
-        generator expression (depending on the value of `iter`).
+    out : {{list, iterator}}
+        The sequence ordered by `index`, as a `list` or as an
+        iterator (depending on the value of `iter`).
 
     See Also
     --------
     index_natsorted
-    index_versorted
     index_humansorted
     index_realsorted
 
@@ -684,7 +572,7 @@ def order_by_index(seq, index, iter=False):
     --------
 
     `order_by_index` is a convenience function that helps you apply
-    the result of `index_natsorted` or `index_versorted`::
+    the result of `index_natsorted`::
 
         >>> a = ['num3', 'num5', 'num2']
         >>> b = ['foo', 'bar', 'baz']
@@ -693,9 +581,179 @@ def order_by_index(seq, index, iter=False):
         [2, 0, 1]
         >>> # Sort both lists by the sort order of a
         >>> order_by_index(a, index)
-        [{u}'num2', {u}'num3', {u}'num5']
+        ['num2', 'num3', 'num5']
         >>> order_by_index(b, index)
-        [{u}'baz', {u}'foo', {u}'bar']
+        ['baz', 'foo', 'bar']
 
     """
     return (seq[i] for i in index) if iter else [seq[i] for i in index]
+
+
+def numeric_regex_chooser(alg):
+    """
+    Select an appropriate regex for the type of number of interest.
+
+    Parameters
+    ----------
+    alg : ns enum
+        Used to indicate the regular expression to select.
+
+    Returns
+    -------
+    regex : str
+        Regular expression string that matches the desired number type.
+
+    """
+    # Remove the leading and trailing parens
+    return utils.regex_chooser(alg).pattern[1:-1]
+
+
+def _split_apply(v, key=None):
+    if key is not None:
+        v = key(v)
+    return utils.path_splitter(str(v))
+
+
+# Choose the implementation based on the host OS
+if platform.system() == "Windows":
+
+    from ctypes import wintypes, windll
+    from functools import cmp_to_key
+
+    _windows_sort_cmp = windll.Shlwapi.StrCmpLogicalW
+    _windows_sort_cmp.argtypes = [wintypes.LPWSTR, wintypes.LPWSTR]
+    _windows_sort_cmp.restype = wintypes.INT
+    _winsort_key = cmp_to_key(_windows_sort_cmp)
+
+    def os_sort_keygen(key=None):
+        return lambda x: tuple(map(_winsort_key, _split_apply(x, key)))
+
+
+else:
+
+    # For UNIX-based platforms, ICU performs MUCH better than locale
+    # at replicating the file explorer's sort order. We will use
+    # ICU's ability to do basic natural sorting as it also better
+    # replicates than what natsort does by default.
+    #
+    # However, if the user does not have ICU installed then fall back
+    # on natsort's default handling for paths with locale turned on
+    # which will give good results in most cases (e.g. when there aren't
+    # a bunch of special characters).
+    try:
+        import icu
+
+    except ImportError:
+        # No ICU installed
+        def os_sort_keygen(key=None):
+            return natsort_keygen(
+                key=key, alg=ns.LOCALE | ns.PATH | ns.IGNORECASE
+            )
+
+    else:
+        # ICU installed
+        def os_sort_keygen(key=None):
+            loc = natsort.compat.locale.get_icu_locale()
+            collator = icu.Collator.createInstance(loc)
+            collator.setAttribute(
+                icu.UCollAttribute.NUMERIC_COLLATION, icu.UCollAttributeValue.ON
+            )
+            return lambda x: tuple(map(collator.getSortKey, _split_apply(x, key)))
+
+
+os_sort_keygen.__doc__ = """
+Generate a sorting key to replicate your file browser's sort order
+
+See :func:`os_sorted` for description and caveats.
+
+Returns
+-------
+out : function
+    A function that parses input for OS path sorting that is
+    suitable for passing as the `key` argument to functions
+    such as `sorted`.
+
+See Also
+--------
+os_sort_key
+os_sorted
+
+Notes
+-----
+On Windows, this will implicitly coerce all inputs to str before
+collating.
+
+"""
+
+os_sort_key = os_sort_keygen()
+os_sort_key.__doc__ = """
+os_sort_key(val)
+The default key to replicate your file browser's sort order
+
+This is the output of :func:`os_sort_keygen` with default values.
+
+See Also
+--------
+os_sort_keygen
+
+"""
+
+
+def os_sorted(seq, key=None, reverse=False):
+    """
+    Sort elements in the same order as your operating system's file browser
+
+    .. warning::
+
+        The resulting function will generate results that will be
+        differnt depending on your platform. This is intentional.
+
+    On Windows, this will sort with the same order as Windows Explorer.
+
+    On MacOS/Linux, you will get different results depending on whether
+    or not you have :mod:`pyicu` installed.
+
+    - If you have :mod:`pyicu` installed, you will get results that are
+      the same as (or very close to) the same order as your operating
+      system's file browser.
+    - If you do not have :mod:`pyicu` installed, then this will give
+      the same results as if you used ``ns.LOCALE``, ``ns.PATH``,
+      and ``ns.IGNORECASE`` with :func:`natsorted`. If you do not have
+      special characters this will give correct results, but once
+      special characters are added you should lower your expectations.
+
+    It is *strongly* reccommended to have :mod:`pyicu` installed on
+    MacOS/Linux if you want correct sort results.
+
+    It does *not* take into account if a path is a directory or a file
+    when sorting.
+
+    Parameters
+    ----------
+    seq : iterable
+        The input to sort. Each element must be of type str.
+
+    key : callable, optional
+        A key used to determine how to sort each element of the sequence.
+        It should accept a single argument and return a single value.
+
+    reverse : {{True, False}}, optional
+        Return the list in reversed sorted order. The default is
+        `False`.
+
+    Returns
+    -------
+    out : list
+        The sorted input.
+
+    See Also
+    --------
+    natsorted
+    os_sort_keygen
+
+    Notes
+    -----
+    This will implicitly coerce all inputs to str before collating.
+
+    """
+    return sorted(seq, key=os_sort_keygen(key), reverse=reverse)
