@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """These test the utils.py functions."""
-from __future__ import unicode_literals
 
+import os
 import pathlib
 import string
 from itertools import chain
@@ -11,12 +11,11 @@ import pytest
 from hypothesis import given
 from hypothesis.strategies import integers, lists, sampled_from, text
 from natsort import utils
-from natsort.compat.py23 import py23_cmp, py23_int, py23_lower, py23_str
 from natsort.ns_enum import ns
 
 
 def test_do_decoding_decodes_bytes_string_to_unicode():
-    assert type(utils.do_decoding(b"bytes", "ascii")) is py23_str
+    assert type(utils.do_decoding(b"bytes", "ascii")) is str
     assert utils.do_decoding(b"bytes", "ascii") == "bytes"
     assert utils.do_decoding(b"bytes", "ascii") == b"bytes".decode("ascii")
 
@@ -100,7 +99,7 @@ def test_groupletters_returns_letters_with_lowercase_transform_of_letter_example
 @given(text().filter(bool))
 def test_groupletters_returns_letters_with_lowercase_transform_of_letter(x):
     assert utils.groupletters(x) == "".join(
-        chain.from_iterable([py23_lower(y), y] for y in x)
+        chain.from_iterable([y.casefold(), y] for y in x)
     )
 
 
@@ -124,44 +123,35 @@ def test_sep_inserter_inserts_separator_between_two_numbers(x):
     result = list(utils.sep_inserter(iter(x), ""))
     for i, pos in enumerate(result[1:-1], 1):
         if pos == "":
-            assert isinstance(result[i - 1], py23_int)
-            assert isinstance(result[i + 1], py23_int)
+            assert isinstance(result[i - 1], int)
+            assert isinstance(result[i + 1], int)
 
 
 def test_path_splitter_splits_path_string_by_separator_example():
-    z = "/this/is/a/path"
-    assert tuple(utils.path_splitter(z)) == tuple(pathlib.Path(z).parts)
-    z = pathlib.Path("/this/is/a/path")
-    assert tuple(utils.path_splitter(z)) == tuple(pathlib.Path(z).parts)
+    given = "/this/is/a/path"
+    expected = (os.sep, "this", "is", "a", "path")
+    assert tuple(utils.path_splitter(given)) == tuple(expected)
+    given = pathlib.Path(given)
+    assert tuple(utils.path_splitter(given)) == tuple(expected)
 
 
 @given(lists(sampled_from(string.ascii_letters), min_size=2).filter(all))
 def test_path_splitter_splits_path_string_by_separator(x):
-    z = py23_str(pathlib.Path(*x))
+    z = str(pathlib.Path(*x))
     assert tuple(utils.path_splitter(z)) == tuple(pathlib.Path(z).parts)
 
 
 def test_path_splitter_splits_path_string_by_separator_and_removes_extension_example():
-    z = "/this/is/a/path/file.exe"
-    y = tuple(pathlib.Path(z).parts)
-    assert tuple(utils.path_splitter(z)) == y[:-1] + (
-        pathlib.Path(z).stem,
-        pathlib.Path(z).suffix,
-    )
+    given = "/this/is/a/path/file.x1.10.tar.gz"
+    expected = (os.sep, "this", "is", "a", "path", "file.x1.10", ".tar", ".gz")
+    assert tuple(utils.path_splitter(given)) == tuple(expected)
 
 
 @given(lists(sampled_from(string.ascii_letters), min_size=3).filter(all))
 def test_path_splitter_splits_path_string_by_separator_and_removes_extension(x):
-    z = py23_str(pathlib.Path(*x[:-2])) + "." + x[-1]
+    z = str(pathlib.Path(*x[:-2])) + "." + x[-1]
     y = tuple(pathlib.Path(z).parts)
     assert tuple(utils.path_splitter(z)) == y[:-1] + (
         pathlib.Path(z).stem,
         pathlib.Path(z).suffix,
     )
-
-
-@given(integers())
-def test_py23_cmp(x):
-    assert py23_cmp(x, x) == 0
-    assert py23_cmp(x, x + 1) < 0
-    assert py23_cmp(x, x - 1) > 0
